(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs/Observable'), require('d3'), require('@angular/core'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define(['exports', 'rxjs/Observable', 'd3', '@angular/core', '@angular/common'], factory) :
	(factory((global.NgCanvasModule = {}),global.Rx,global.d3,global.ng.core,global.common));
}(this, (function (exports,Observable,d3,core,common) { 'use strict';

(function (d3_util) {
    function getRect(selector) {
        return selector.node().getBoundingClientRect();
    }
    d3_util.getRect = getRect;
    function createGroup(tag) {
        if (tag === void 0) { tag = 'g'; }
        return d3.select(document.createElementNS(d3.namespaces.svg, tag));
    }
    d3_util.createGroup = createGroup;
    function createEqTriangle(sideLength, centerPosition) {
        var pi = 3.141592653589793238462643383;
        var halfSide = sideLength / 2;
        var innerHypotenuse = halfSide * (1 / Math.cos(30 * pi / 180));
        var innerOpposite = halfSide * (1 / Math.tan(60 * pi / 180));
        var left = [centerPosition[0] - halfSide, centerPosition[1] + innerOpposite];
        var right = [centerPosition[0] + halfSide, centerPosition[1] + innerOpposite];
        var top = [centerPosition[0], centerPosition[1] - innerHypotenuse];
        return [top, left, right];
    }
    d3_util.createEqTriangle = createEqTriangle;
})(exports.d3_util || (exports.d3_util = {}));

(function (SVG_ELEMENT_TYPE) {
    SVG_ELEMENT_TYPE["CANVAS"] = "Canvas";
    SVG_ELEMENT_TYPE["AXIS_X"] = "AxisX";
    SVG_ELEMENT_TYPE["AXIS_Y"] = "AxisY";
    SVG_ELEMENT_TYPE["LEGEND"] = "Legend";
    SVG_ELEMENT_TYPE["PATH"] = "Path";
})(exports.SVG_ELEMENT_TYPE || (exports.SVG_ELEMENT_TYPE = {}));
var SvgElement = (function () {
    function SvgElement(type, name) {
        if (name === void 0) { name = ''; }
        this.children = new Array();
        this.attrs = {};
        this.styles = {};
        this.type = type;
        this.name = name;
        this.group = exports.d3_util.createGroup();
    }
    SvgElement.prototype.setAttr = function (name, value) {
        this.attrs[name] = value;
        return this;
    };
    SvgElement.prototype.delAttr = function (name) {
        delete this.attrs[name];
    };
    SvgElement.prototype.setStyle = function (name, value) {
        this.styles[name] = value;
        return this;
    };
    SvgElement.prototype.getStyle = function (name) {
        return this.styles[name];
    };
    SvgElement.prototype.delStyle = function (name) {
        delete this.styles[name];
    };
    SvgElement.prototype.addChild = function (child) {
        if (child) {
            child.parent = this;
            this.children.push(child);
        }
        return this;
    };
    SvgElement.prototype.applyStyle = function () {
        var _this = this;
        Object.keys(this.styles).forEach(function (k) { return _this.group.style(k, _this.styles[k]); });
        Object.keys(this.attrs).forEach(function (k) { return _this.group.attr(k, _this.attrs[k]); });
    };
    SvgElement.prototype.clearAll = function () {
        this.group.selectAll('*').remove();
        this.group.remove();
    };
    SvgElement.prototype.clear = function () {
        this.group.selectAll('*').remove();
    };
    SvgElement.prototype.rebuild = function (child) {
        child.clearAll();
        this.group.append(function () { return child.group.node(); });
        child.buildGroup();
    };
    return SvgElement;
}());
var Rect = {
    left: function (o) { return o ? o.left || 0 : 0; },
    top: function (o) { return o ? o.top || 0 : 0; },
    right: function (o) { return o ? o.right || 0 : 0; },
    bottom: function (o) { return o ? o.bottom || 0 : 0; },
    width: function (o) { return o ? o.width || 0 : 0; },
    height: function (o) { return o ? o.height || 0 : 0; }
};

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Axis = (function (_super) {
    __extends$1(Axis, _super);
    function Axis(type, name, unit, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, type, name) || this;
        _this.defaultOptions = {
            tickInterval: 0,
            extraSpace: 0,
            disableAxisTitle: false,
            disableAxisLine: false,
            disableTickLine: false,
            disableTickMark: false,
            disableTickNumber: false
        };
        _this.options = {};
        _this.domain = [0, 0];
        _this.unit = unit;
        Object.assign(_this.options, options);
        return _this;
    }
    Axis.prototype.setRange = function (r) {
        this.range = r;
    };
    Axis.prototype.setDomain = function (domain) {
        this.domain[0] = Math.min(this.domain[0], domain[0]);
        this.domain[1] = Math.max(this.domain[1], domain[1]);
        this.validDomain();
        this.calculateTickValues();
    };
    Axis.prototype.resetDomainWithPoint = function (first, last) {
        var min, max;
        if (this.type === exports.SVG_ELEMENT_TYPE.AXIS_X) {
            min = first.x;
            max = last.x + (this.options.extraSpace || 0);
        }
        else {
            min = first.y;
            max = last.y + (this.options.extraSpace || 0);
        }
        max = Math.floor(max + 0.5);
        this.setDomain([min, max]);
    };
    Object.defineProperty(Axis.prototype, "title", {
        get: function () {
            var title = this.name;
            if (this.unit && this.unit !== '') {
                title = title + " (" + this.unit + ")";
            }
            return title;
        },
        enumerable: true,
        configurable: true
    });
    Axis.prototype.getScale = function () {
        return this.scale;
    };
    Axis.prototype.applyAxisOptions = function (options) {
        this.options = Object.assign({}, options, this.options);
    };
    Axis.prototype.calculateTickValues = function () {
        var domain = this.domain;
        var tickInterval = this.options.tickInterval;
        if (domain) {
            if (tickInterval && tickInterval > 0) {
                this.tickValues = new Array();
                var i = domain[0];
                for (; i <= domain[1]; i = i + tickInterval) {
                    this.tickValues.push(i);
                }
                if (i > domain[1]) {
                    this.tickValues.push(domain[1]);
                }
            }
        }
    };
    Axis.prototype.validDomain = function () {
        var domain = this.domain;
        if (domain && domain[0] > domain[1]) {
            this.domain = (_a = [domain[1], domain[0]], domain[0] = _a[0], domain[1] = _a[1], _a);
        }
        var _a;
    };
    Axis.prototype.createAxisAndScale = function () {
        // scale
        this.scale = d3.scaleLinear()
            .domain(this.domain)
            .range(this.range);
        // axis
        this.axis = this.type === exports.SVG_ELEMENT_TYPE.AXIS_X
            ? d3.axisBottom(this.scale)
            : d3.axisLeft(this.scale);
        // tick
        if (this.tickValues && this.tickValues.length > 0) {
            this.axis.tickValues(this.tickValues);
        }
    };
    Axis.prototype.appendAxisTitle = function () {
        var parentSize = this.parent.size;
        var className = this.type.substr(-1).toLocaleLowerCase() + "-axis-title";
        var text = this.group
            .append('text')
            .attr('class', "axis-title " + className)
            .text(this.title);
        if (this.type === exports.SVG_ELEMENT_TYPE.AXIS_X) {
            text
                .attr('x', Rect.width(parentSize))
                .attr('y', -3)
                .attr('dy', '-.35em')
                .attr('fill', '#000')
                .style('text-anchor', 'end');
        }
        else {
            text
                .attr('dx', 8)
                .attr('dy', '.8em')
                .attr('fill', '#000')
                .style('text-anchor', 'start');
        }
    };
    Axis.prototype.appendTickLine = function (noZeroTick, hasLastTick) {
        if (noZeroTick === void 0) { noZeroTick = true; }
        if (hasLastTick === void 0) { hasLastTick = true; }
        var parentSize = this.parent.size;
        var tickClass = noZeroTick ? '.tick:not(:first-of-type)' : '.tick';
        var lines = this.group.selectAll(tickClass)
            .append('line')
            .attr('stroke', '#777')
            .attr('stroke-dasharray', '2,2');
        if (this.type === exports.SVG_ELEMENT_TYPE.AXIS_X) {
            lines.attr('y2', -Rect.height(parentSize));
        }
        else {
            lines.attr('x2', Rect.width(parentSize));
        }
    };
    Axis.prototype.buildGroup = function () {
        this.createAxisAndScale();
        var parentSize = this.parent.size;
        var className = this.type.substr(-1).toLocaleLowerCase() + "-axis";
        var noZeroTick = true;
        this.group
            .attr('class', "axis " + className)
            .call(this.axis);
        if (this.type === exports.SVG_ELEMENT_TYPE.AXIS_X) {
            this.group.attr('transform', 'translate(0,' + parentSize.height + ')');
        }
        if (!this.options.disableAxisTitle) {
            this.appendAxisTitle();
        }
        if (this.options.disableAxisLine) {
            this.disableAxisLine();
            this.disableTickMark();
            noZeroTick = false;
        }
        if (this.options.disableTickMark) {
            this.disableTickMark();
        }
        if (!!this.options.disableTickNumber) {
            this.disableTickNumber();
        }
        if (!this.options.disableTickLine) {
            this.appendTickLine(noZeroTick);
        }
    };
    Axis.prototype.getSize = function () {
        return exports.d3_util.getRect(this.group);
    };
    Axis.prototype.disableAxisLine = function () {
        this.group.select('path').attr('stroke', 'none');
    };
    Axis.prototype.disableTickMark = function () {
        this.group.selectAll('.tick').select('line').attr('stroke', 'none');
    };
    Axis.prototype.disableTickNumber = function () {
        this.group.selectAll('.tick').select('text').attr('fill', 'none');
    };
    return Axis;
}(SvgElement));

/**
 * Copyright (C) 2015 Ivan Kuckir
 * https://github.com/kuckir/CSPL.js.git
 */
var CSPL = (function () {
    function CSPL() {
    }
    CSPL.getNaturalKs = function (xs, ys, ks) {
        var n = xs.length - 1;
        var A = zerosMat(n + 1, n + 2);
        for (var i = 1; i < n; i++) {
            A[i][i - 1] = 1 / (xs[i] - xs[i - 1]);
            A[i][i] = 2 * (1 / (xs[i] - xs[i - 1]) + 1 / (xs[i + 1] - xs[i]));
            A[i][i + 1] = 1 / (xs[i + 1] - xs[i]);
            A[i][n + 1] = 3 * ((ys[i] - ys[i - 1]) / ((xs[i] - xs[i - 1]) * (xs[i] - xs[i - 1]))
                + (ys[i + 1] - ys[i]) / ((xs[i + 1] - xs[i]) * (xs[i + 1] - xs[i])));
        }
        A[0][0] = 2 / (xs[1] - xs[0]);
        A[0][1] = 1 / (xs[1] - xs[0]);
        A[0][n + 1] = 3 * (ys[1] - ys[0]) / ((xs[1] - xs[0]) * (xs[1] - xs[0]));
        A[n][n - 1] = 1 / (xs[n] - xs[n - 1]);
        A[n][n] = 2 / (xs[n] - xs[n - 1]);
        A[n][n + 1] = 3 * (ys[n] - ys[n - 1]) / ((xs[n] - xs[n - 1]) * (xs[n] - xs[n - 1]));
        solve(A, ks);
    };
    CSPL.evalSpline = function (x, xs, ys, ks) {
        var i = 1;
        while (xs[i] < x) {
            i++;
        }
        var t = (x - xs[i - 1]) / (xs[i] - xs[i - 1]);
        var a = ks[i - 1] * (xs[i] - xs[i - 1]) - (ys[i] - ys[i - 1]);
        var b = -ks[i] * (xs[i] - xs[i - 1]) + (ys[i] - ys[i - 1]);
        return (1 - t) * ys[i - 1] + t * ys[i] + t * (1 - t) * (a * (1 - t) + b * t);
    };
    return CSPL;
}());
// in Matrix, out solutions
function solve(A, x) {
    var m = A.length;
    for (var k = 0; k < m; k++) {
        // pivot for column
        var i_max = 0;
        var vali = Number.NEGATIVE_INFINITY;
        for (var i = k; i < m; i++) {
            if (Math.abs(A[i][k]) > vali) {
                i_max = i;
                vali = Math.abs(A[i][k]);
            }
        }
        if (k !== i_max) {
            swapRows(A, k, i_max);
        }
        // for all rows below pivot
        for (var i = k + 1; i < m; i++) {
            var cf = (A[i][k] / A[k][k]);
            for (var j = k; j < m + 1; j++) {
                A[i][j] -= A[k][j] * cf;
            }
        }
    }
    // rows = columns
    for (var i = m - 1; i >= 0; i--) {
        var v = A[i][m] / A[i][i];
        x[i] = v;
        // rows
        for (var j = i - 1; j >= 0; j--) {
            A[j][m] -= A[j][i] * v;
            A[j][i] = 0;
        }
    }
}
function swapRows(m, k, l) {
    var p = m[k];
    m[k] = m[l];
    m[l] = p;
}
function zerosMat(r, c) {
    var A = [];
    for (var i = 0; i < r; i++) {
        A.push([]);
        for (var j = 0; j < c; j++) {
            A[i].push(0);
        }
    }
    return A;
}

var Point = (function () {
    function Point(x, y, color) {
        if (color === void 0) { color = ''; }
        this.x = x;
        this.y = y;
        this.pointShape = null;
        this.color = color;
    }
    Point.prototype.createElement = function (canvas, color, pointShape) {
        var element;
        var x = canvas.xScale(this.x);
        var y = canvas.yScale(this.y);
        pointShape = this.pointShape || pointShape;
        color = this.color || color;
        if (pointShape === exports.PointShape.SQUARE) {
            var squareLength = 5;
            element = exports.d3_util.createGroup('rect')
                .attr('x', x - squareLength / 2).attr('y', y - squareLength / 2)
                .attr('width', squareLength).attr('height', squareLength);
        }
        else if (pointShape === exports.PointShape.TRIANGLE) {
            var sideLength = 8;
            var pts = exports.d3_util.createEqTriangle(sideLength, [x, y]);
            element = exports.d3_util.createGroup('polygon')
                .attr('points', pts.map(function (p) { return p.join(','); }).join(' '));
        }
        else {
            var radius = 3;
            element = exports.d3_util.createGroup('circle')
                .attr('cx', x)
                .attr('cy', y)
                .attr('r', radius);
        }
        element.attr('fill', color).attr('stroke', 'transparent');
        return element;
    };
    return Point;
}());

(function (PointShape) {
    PointShape["CIRCLE"] = "Circle";
    PointShape["TRIANGLE"] = "Triangle";
    PointShape["SQUARE"] = "Square";
})(exports.PointShape || (exports.PointShape = {}));

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();

(function (PathType) {
    PathType["CURVED_LINE"] = "CurvedLine";
    PathType["STRAIGHT_LINE"] = "StraightLine";
})(exports.PathType || (exports.PathType = {}));
var Path = (function (_super) {
    __extends$2(Path, _super);
    function Path(points, name) {
        if (name === void 0) { name = ''; }
        var _this = _super.call(this, exports.SVG_ELEMENT_TYPE.PATH, name) || this;
        _this.axisStep = 1;
        _this.hasDot = true;
        _this.pathType = exports.PathType.CURVED_LINE;
        _this.pointShape = exports.PointShape.CIRCLE;
        _this.xRef = {};
        _this.yRef = {};
        _this.points = points;
        _this.setDefaultStyle();
        _this.sort();
        return _this;
    }
    Path.prototype.sort = function () {
        var points = this.points.slice();
        this.points.sort(function (a, b) { return a.x - b.x; });
        points.sort(function (a, b) { return a.y - b.y; });
        this.xRef.first = this.points[0];
        this.xRef.last = this.points[this.points.length - 1];
        this.yRef.first = points[0];
        this.yRef.last = points[points.length - 1];
    };
    Object.defineProperty(Path.prototype, "color", {
        get: function () {
            return this.getStyle('stroke');
        },
        set: function (color) {
            this.setStyle('stroke', color);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "xFirst", {
        get: function () {
            return this.xRef.first;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "xLast", {
        get: function () {
            return this.xRef.last;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "yFirst", {
        get: function () {
            return this.yRef.first;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "yLast", {
        get: function () {
            return this.yRef.last;
        },
        enumerable: true,
        configurable: true
    });
    Path.prototype.setPoints = function (points) {
        this.points = points;
        this.sort();
    };
    Path.prototype.setDefaultStyle = function () {
        this.setAttr('class', 'line')
            .setStyle('stroke', '#000')
            .setAttr('fill', 'transparent')
            .setAttr('stroke-width', '1.5px');
    };
    Path.prototype.buildGroup = function () {
        var _this = this;
        this.applyStyle();
        var path;
        if (this.pathType === exports.PathType.CURVED_LINE) {
            path = this._buildCurvedPath();
        }
        else {
            path = this.points.map(function (p) { return _this._toXY(p.x, p.y); });
        }
        this.group.append('path').attr('d', path.reduce(function (s, p) { return s += p.join(',') + ' '; }, 'M').trim());
        if (this.hasDot) {
            this.points.forEach(function (p) { return _this.group.append(function () { return p.createElement(_this.parent, _this.color, _this.pointShape).node(); }); });
        }
    };
    Path.prototype._buildCurvedPath = function () {
        var path = [];
        var step = 2 / this.parent.xScale(this.axisStep);
        var points = this.points;
        var xs = [], ys = [], ks = [];
        points.forEach(function (p, i) {
            xs[i] = p.x;
            ys[i] = p.y;
            ks[i] = 1;
        });
        CSPL.getNaturalKs(xs, ys, ks);
        var minx = points[0].x;
        var maxx = points[points.length - 1].x;
        path.push(this._toXY(minx, CSPL.evalSpline(minx, xs, ys, ks)));
        for (var i = minx + step; i <= maxx; i += step) {
            path.push(this._toXY(i, CSPL.evalSpline(i, xs, ys, ks)));
        }
        return path;
    };
    Path.prototype._toXY = function (x, y) {
        return [this.parent.xScale(x), this.parent.yScale(y)];
    };
    return Path;
}(SvgElement));

var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var PositionH;
(function (PositionH) {
    PositionH["LEFT"] = "Left";
    PositionH["RIGHT"] = "Right";
    PositionH["CENTER"] = "Center";
})(PositionH || (PositionH = {}));
var PositionV;
(function (PositionV) {
    PositionV["UP"] = "Up";
    PositionV["BOTTOM"] = "Bottom";
    PositionV["MIDDLE"] = "Middle";
})(PositionV || (PositionV = {}));

(function (LegendLayout) {
    LegendLayout["BLOCK"] = "Block";
    LegendLayout["LINE"] = "Line";
})(exports.LegendLayout || (exports.LegendLayout = {}));

(function (LegendPosition) {
    LegendPosition["UP_LEFT"] = "UpLeft";
    LegendPosition["UP_RIGHT"] = "UpRight";
    LegendPosition["BOTTOM_LEFT"] = "BottomLeft";
    LegendPosition["BOTTOM_RIGHT"] = "BottomRight";
    LegendPosition["UP_CENTER"] = "UpCenter";
    LegendPosition["BOTTOM_CENTER"] = "BottomCenter";
})(exports.LegendPosition || (exports.LegendPosition = {}));

(function (LegendRange) {
    LegendRange["IN"] = "In";
    LegendRange["OUT"] = "Out";
    LegendRange["SIDE"] = "Side";
})(exports.LegendRange || (exports.LegendRange = {}));

(function (LegendShape) {
    LegendShape["LINE"] = "Line";
    LegendShape["CIRCLE"] = "Circle";
    LegendShape["RECTANGLE"] = "Rectangle";
    LegendShape["ELLIPSE"] = "Ellipse";
})(exports.LegendShape || (exports.LegendShape = {}));
var yAxisWidth = 20;
var Legend = (function (_super) {
    __extends$3(Legend, _super);
    function Legend(layout, position, range) {
        var _this = _super.call(this, exports.SVG_ELEMENT_TYPE.LEGEND) || this;
        _this.layout = layout;
        _this.position = position;
        _this.range = range;
        _this.legendShape = exports.LegendShape.LINE;
        /**
         * Legend margin, can be changed
         */
        _this.insideRect = { top: 30, right: 25, bottom: 20, left: 30 };
        var ps = position.split(/(?=[A-Z])/);
        _this.positionV = PositionV[ps[0].toUpperCase()];
        _this.positionH = PositionH[ps[1].toUpperCase()];
        return _this;
    }
    Object.defineProperty(Legend.prototype, "$legendSize", {
        get: function () {
            return this.legendSize;
        },
        enumerable: true,
        configurable: true
    });
    Legend.prototype.buildGroup = function () {
        this.group
            .attr('class', 'legend')
            .style('fill', 'transparent')
            .style('stroke-width', '1px');
    };
    Legend.prototype.itemX = function (i, offset) {
        switch (this.layout) {
            case exports.LegendLayout.LINE:
                return 15 * i + offset;
            case exports.LegendLayout.BLOCK:
            default:
                return 0;
        }
    };
    Legend.prototype.itemY = function (i) {
        switch (this.layout) {
            case exports.LegendLayout.LINE:
                return 0;
            case exports.LegendLayout.BLOCK:
            default:
                return 15 * i;
        }
    };
    Legend.prototype.x = function (size, fullSize, titleSize) {
        if (this.range === exports.LegendRange.IN || this.range === exports.LegendRange.OUT) {
            switch (this.positionH) {
                case PositionH.RIGHT:
                    return Rect.width(fullSize) - this.legendSize.width - Rect.right(this.insideRect);
                case PositionH.CENTER:
                    return (Rect.width(size) - this.legendSize.width) / 2 + Rect.left(this.insideRect) + yAxisWidth;
                case PositionH.LEFT:
                default:
                    return Rect.left(this.insideRect) + yAxisWidth;
            }
        }
        else {
            // LegendRange.SIDE
            switch (this.positionH) {
                case PositionH.RIGHT:
                    return Rect.width(size) + this.legendSize.width / 2;
                case PositionH.LEFT:
                default:
                    return yAxisWidth;
            }
        }
    };
    Legend.prototype.y = function (size, fullSize, titleSize) {
        if (this.range === exports.LegendRange.IN || this.range === exports.LegendRange.SIDE) {
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    return Rect.height(size) - this.legendSize.height;
                case PositionV.UP:
                default:
                    return this.insideRect.top + Rect.height(titleSize);
            }
        }
        else {
            // LegendRange.OUT
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    return Rect.height(fullSize) - this.legendSize.height - 10;
                case PositionV.UP:
                default:
                    return Rect.height(titleSize) + Rect.top(this.insideRect) / 2;
            }
        }
    };
    Legend.prototype.getCoordinateMargin = function (coordinateMargin) {
        var margin = Object.assign({}, coordinateMargin);
        if (this.range === exports.LegendRange.OUT) {
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    margin.bottom = margin.bottom || 0 + 30;
                    break;
                case PositionV.UP:
                default:
                    margin.top = margin.top || 0 + 30;
            }
        }
        else if (this.range === exports.LegendRange.SIDE) {
            switch (this.positionH) {
                case PositionH.RIGHT:
                    margin.right += this.legendSize.width + Rect.right(this.insideRect);
                    break;
                case PositionH.LEFT:
                default:
                    margin.left += this.legendSize.width + Rect.left(this.insideRect);
            }
        }
        return margin;
    };
    Legend.prototype.createLegendItem = function () {
        var _this = this;
        this.clear();
        var offset = 0;
        return {
            loop: function (name, color, index) {
                var legendItem = exports.d3_util.createGroup().attr('class', 'legend-item');
                var figure = _this.createItemFigure(color);
                _this.group.append(function () { return legendItem.node(); });
                legendItem.append(function () { return figure.node(); });
                var figureRect = exports.d3_util.getRect(figure);
                legendItem.append('text')
                    .attr('x', _this.getItemX(figureRect))
                    .attr('y', figureRect.height === 0 ? 3 : figureRect.height / 2)
                    .attr('fill', '#000')
                    .style('font', '10px sans-serif')
                    .text(name);
                var xItem = _this.itemX(index, offset);
                var yItem = _this.itemY(index);
                legendItem.attr('transform', "translate(" + xItem + "," + yItem + ")");
                offset += exports.d3_util.getRect(legendItem).width;
                return legendItem;
            },
            end: function () {
                _this.legendSize = exports.d3_util.getRect(_this.group);
            }
        };
    };
    Legend.prototype.transform = function (size, fullSize, titleSize) {
        var xLegend = this.x(size, fullSize, titleSize);
        var yLegend = this.y(size, fullSize, titleSize);
        this.group.attr('transform', "translate(" + xLegend + "," + yLegend + ")");
        this.legendSize = exports.d3_util.getRect(this.group);
    };
    Legend.prototype.createItemFigure = function (color) {
        switch (this.legendShape) {
            case exports.LegendShape.CIRCLE:
                return exports.d3_util.createGroup('circle')
                    .attr('cy', 1)
                    .attr('r', 4)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case exports.LegendShape.RECTANGLE:
                return exports.d3_util.createGroup('rect')
                    .attr('y', -3)
                    .attr('width', 10)
                    .attr('height', 5)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case exports.LegendShape.ELLIPSE:
                return exports.d3_util.createGroup('ellipse')
                    .attr('y', -2)
                    .attr('rx', 8)
                    .attr('ry', 4)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case exports.LegendShape.LINE:
            default:
                return exports.d3_util.createGroup('line')
                    .attr('x2', 10)
                    .attr('y2', 0)
                    .style('stroke', color)
                    .style('fill', 'transparent')
                    .style('stroke-width', '1px');
        }
    };
    Legend.prototype.getItemX = function (figureRect) {
        switch (this.legendShape) {
            case exports.LegendShape.CIRCLE:
            case exports.LegendShape.ELLIPSE:
                return figureRect.width / 2 + 3;
            case exports.LegendShape.RECTANGLE:
            case exports.LegendShape.LINE:
            default:
                return figureRect.width + 3;
        }
    };
    return Legend;
}(SvgElement));

(function (LegendStyle) {
    // in & up
    LegendStyle.UP_LEFT_IN_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.UP_LEFT, exports.LegendRange.IN);
    LegendStyle.UP_LEFT_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_LEFT, exports.LegendRange.IN);
    LegendStyle.UP_RIGHT_IN_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.UP_RIGHT, exports.LegendRange.IN);
    LegendStyle.UP_RIGHT_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_RIGHT, exports.LegendRange.IN);
    // in & bottom
    LegendStyle.BOTTOM_LEFT_IN_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.BOTTOM_LEFT, exports.LegendRange.IN);
    LegendStyle.BOTTOM_LEFT_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.BOTTOM_LEFT, exports.LegendRange.IN);
    LegendStyle.BOTTOM_RIGHT_IN_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.BOTTOM_RIGHT, exports.LegendRange.IN);
    LegendStyle.BOTTOM_RIGHT_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.BOTTOM_RIGHT, exports.LegendRange.IN);
    LegendStyle.UP_CENTER_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_CENTER, exports.LegendRange.IN);
    LegendStyle.BOTTOM_CENTER_IN_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.BOTTOM_CENTER, exports.LegendRange.IN);
    // out & line
    LegendStyle.UP_CENTER_OUT_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_CENTER, exports.LegendRange.OUT);
    LegendStyle.UP_LEFT_OUT_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_LEFT, exports.LegendRange.OUT);
    LegendStyle.UP_RIGHT_OUT_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.UP_RIGHT, exports.LegendRange.OUT);
    LegendStyle.BOTTOM_CENTER_OUT_LINE = new Legend(exports.LegendLayout.LINE, exports.LegendPosition.BOTTOM_CENTER, exports.LegendRange.OUT);
    // side & block
    LegendStyle.UP_LEFT_SIDE_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.UP_LEFT, exports.LegendRange.SIDE);
    LegendStyle.BOTTOM_LEFT_SIDE_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.BOTTOM_LEFT, exports.LegendRange.SIDE);
    LegendStyle.UP_RIGHT_SIDE_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.UP_RIGHT, exports.LegendRange.SIDE);
    LegendStyle.BOTTOM_RIGHT_SIDE_BLOCK = new Legend(exports.LegendLayout.BLOCK, exports.LegendPosition.BOTTOM_RIGHT, exports.LegendRange.SIDE);
})(exports.LegendStyle || (exports.LegendStyle = {}));

(function (CanvasStyle) {
    CanvasStyle["Coordinate"] = "Coordinate";
    CanvasStyle["CanvasBoard"] = "CanvasBoard";
})(exports.CanvasStyle || (exports.CanvasStyle = {}));
var CanvasStyles = {
    Coordinate: {
        CanvasOptions: {
            hasTitle: true,
            hasLegend: true
        },
        AxisOptions: {
            tickInterval: 0,
            extraSpace: 0,
            disableAxisTitle: false,
            disableAxisLine: false,
            disableTickLine: false,
            disableTickMark: false,
            disableTickNumber: false,
        }
    },
    CanvasBoard: {
        CanvasOptions: {
            hasTitle: false,
            hasLegend: false
        },
        AxisOptions: {
            tickInterval: 0,
            extraSpace: 0,
            disableAxisTitle: true,
            disableAxisLine: true,
            disableTickLine: false,
            disableTickMark: true,
            disableTickNumber: true,
        }
    }
};

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DEFAULT_MARGIN = { top: 5, left: 30, right: 15, bottom: 20 };
var Canvas = (function (_super) {
    __extends(Canvas, _super);
    function Canvas(name, fullSize, canvasStyle, domain, legend) {
        if (canvasStyle === void 0) { canvasStyle = exports.CanvasStyle.Coordinate; }
        if (domain === void 0) { domain = [0, 0]; }
        if (legend === void 0) { legend = exports.LegendStyle.UP_LEFT_IN_BLOCK; }
        var _this = _super.call(this, exports.SVG_ELEMENT_TYPE.CANVAS, name) || this;
        /**
       * Canvas margin, can be changed
       */
        _this.margin = { top: 0, left: 0, right: 0, bottom: 0 };
        _this.defaultMargin = DEFAULT_MARGIN;
        _this.fullSize = { width: 0, height: 0 };
        _this.canvasOptions = {};
        _this.paths = new Array();
        _this.xyAxis = {};
        _this._hasObservable = false;
        // TODO
        _this.runningObservable = 0;
        // this.legend = styles[10];
        _this.size = { width: 0, height: 0 };
        _this.fullSize = fullSize;
        _this.canvasStyle = canvasStyle || exports.CanvasStyle.Coordinate;
        _this.styleOptions = CanvasStyles[_this.canvasStyle];
        _this.legend = legend;
        _this.canvasDomain = domain;
        _this.titleGroup = exports.d3_util.createGroup();
        return _this;
    }
    Object.defineProperty(Canvas.prototype, "hasObservable", {
        get: function () {
            return this._hasObservable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "xAxis", {
        get: function () {
            if (!this.xyAxis[exports.SVG_ELEMENT_TYPE.AXIS_X]) {
                this.xAxis = this.createX('', '', {});
            }
            return this.xyAxis[exports.SVG_ELEMENT_TYPE.AXIS_X];
        },
        set: function (xAxis) {
            if (xAxis) {
                xAxis.type = exports.SVG_ELEMENT_TYPE.AXIS_X;
                this.setXyAxis(xAxis);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "yAxis", {
        get: function () {
            if (!this.xyAxis[exports.SVG_ELEMENT_TYPE.AXIS_Y]) {
                this.yAxis = this.createY('', '', {});
            }
            return this.xyAxis[exports.SVG_ELEMENT_TYPE.AXIS_Y];
        },
        set: function (yAxis) {
            if (yAxis) {
                yAxis.type = exports.SVG_ELEMENT_TYPE.AXIS_Y;
                this.setXyAxis(yAxis);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "legendShape", {
        /**
         * set legend shape
         * @memberof Canvas
         */
        set: function (legendShape) {
            this.legend.legendShape = legendShape;
        },
        enumerable: true,
        configurable: true
    });
    Canvas.prototype.getX = function () {
        return this.xAxis;
    };
    Canvas.prototype.getY = function () {
        return this.yAxis;
    };
    Canvas.prototype.createX = function (name, unit, options) {
        return this.xAxis = new Axis(exports.SVG_ELEMENT_TYPE.AXIS_X, name, unit, options);
    };
    Canvas.prototype.createY = function (name, unit, options) {
        return this.yAxis = new Axis(exports.SVG_ELEMENT_TYPE.AXIS_Y, name, unit, options);
    };
    Canvas.prototype.setXyAxis = function (axis) {
        this.xyAxis[axis.type] = axis;
        this.addChild(axis);
    };
    /**
     * @public
     * @param {(Path | Point[] | [number, number][] | Observable<[number, number][] | {x: number, y: number}[])} data
     * @returns {Path | null}
     * @memberof Canvas
     * @see Path, Point
     */
    Canvas.prototype.addPath = function (data, name) {
        if (data instanceof Path) {
            return this.addPathFromPath(data);
        }
        else if (data instanceof Array && data.length > 0) {
            if (data[0] instanceof Point) {
                return this.addPathFromPoints(data, name);
            }
            else if (data[0] instanceof Array) {
                return this.addPathFromArray(data, name);
            }
        }
        else if (data instanceof Observable.Observable) {
            return this._addPathFromObservable(data, name);
        }
        return null;
    };
    Canvas.prototype.addPathFromArray = function (data, name) {
        return this.addPathFromPoints(data.map(function (d) { return new Point(d[0], d[1]); }), name);
    };
    Canvas.prototype.addPathFromPoints = function (points, name) {
        return this.addPathFromPath(new Path(points, name));
    };
    Canvas.prototype.addPathFromPath = function (path) {
        this.paths.push(path);
        this.addChild(path);
        return path;
    };
    Canvas.prototype._addPathFromObservable = function (data, name) {
        var _this = this;
        this._hasObservable = true;
        // TODO
        this.runningObservable++;
        var path = new Path([], name);
        data.subscribe(function (d) {
            var points;
            if (d[0] instanceof Array) {
                points = d.map(function (p) { return new Point(p[0], p[1]); });
            }
            else {
                points = d.map(function (p) { return new Point(p.x, p.y); });
            }
            path.setPoints(points);
            _this.addPathFromPath(path);
            // TODO
            _this.runningObservable--;
            if (_this.runningObservable === 0) {
                _this.buildGroup();
            }
        });
        return path;
    };
    Canvas.prototype.xScale = function (value) {
        return this.xAxis.getScale()(value);
    };
    Canvas.prototype.yScale = function (value) {
        return this.yAxis.getScale()(value);
    };
    Canvas.prototype.appendTo = function (host) {
        this.host = host.insert('svg');
        this.setHostSize();
        this.appendGroup();
    };
    Canvas.prototype.setHostSize = function () {
        this.host.attr('width', Rect.width(this.fullSize))
            .attr('height', Rect.height(this.fullSize));
    };
    Canvas.prototype.applyCanvasStyle = function () {
        this.canvasOptions = Object.assign({}, this.styleOptions.CanvasOptions, this.canvasOptions);
    };
    Canvas.prototype.appendGroup = function () {
        var _this = this;
        this.applyCanvasStyle();
        this.host.append(function () { return _this.group.node(); });
        if (this.canvasOptions.hasLegend) {
            this.host.append(function () { return _this.legend.group.node(); });
        }
        if (this.canvasOptions.hasTitle) {
            this.host.append(function () { return _this.titleGroup.node(); });
        }
    };
    Canvas.prototype.buildGroup = function () {
        var _this = this;
        var xAxis = this.xAxis;
        var yAxis = this.yAxis;
        this.applyCanvasStyle();
        // draw title
        if (this.canvasOptions.hasTitle) {
            this.buildTitle();
        }
        // draw legend items
        if (this.canvasOptions.hasLegend) {
            var legendItem_1 = this.legend.createLegendItem();
            this.paths.forEach(function (p, i) { return legendItem_1.loop(p.name, p.color, i); });
            legendItem_1.end();
        }
        // calculate margin
        var margin = this.canvasOptions.hasLegend ? this.legend.getCoordinateMargin(this.margin) : this.margin;
        var defMargin = this.defaultMargin;
        var titleRect = exports.d3_util.getRect(this.titleGroup);
        var chartMargin = {
            left: Rect.left(defMargin) + Rect.left(margin),
            top: Rect.top(defMargin) + Rect.top(margin) + titleRect.height,
            right: Rect.right(defMargin) + Rect.right(margin),
            bottom: Rect.bottom(defMargin) + Rect.bottom(margin)
        };
        // move canvas
        this.group.attr('transform', "translate(" + chartMargin.left + "," + chartMargin.top + ")");
        // calculate canvas size
        this.size.width = Rect.width(this.fullSize) - chartMargin.left - chartMargin.right;
        this.size.height = Rect.height(this.fullSize) - chartMargin.top - chartMargin.bottom;
        xAxis.applyAxisOptions(this.styleOptions.AxisOptions);
        yAxis.applyAxisOptions(this.styleOptions.AxisOptions);
        // set x and y axis
        xAxis.setRange([0, this.size.width]);
        xAxis.setDomain([0, this.canvasDomain[0]]);
        yAxis.setRange([this.size.height, 0]);
        yAxis.setDomain([0, this.canvasDomain[1]]);
        this.paths.forEach(function (p) {
            xAxis.resetDomainWithPoint(p.xFirst, p.xLast);
            yAxis.resetDomainWithPoint(p.yFirst, p.yLast);
        });
        // draw x-axis, y-axis
        [xAxis, yAxis].forEach(function (axis) { return _this.rebuild(axis); });
        // draw paths
        this.paths.forEach(function (path) { return _this.rebuild(path); });
        // Title position
        if (this.canvasOptions.hasTitle) {
            var xTitle = chartMargin.left + (Rect.width(this.size) - titleRect.width) / 2;
            this.titleGroup.attr('transform', "translate(" + xTitle + "," + titleRect.height + ")");
        }
        // Legend position
        if (this.canvasOptions.hasLegend) {
            this.legend.transform(this.size, this.fullSize, titleRect);
        }
    };
    Canvas.prototype.buildTitle = function () {
        // build title
        this.titleGroup.selectAll('*').remove();
        this.titleGroup.append('text')
            .attr('fill', '#000')
            .style('font', '18px sans-serif')
            .text(this.name);
    };
    return Canvas;
}(SvgElement));

var NgCanvasComponent = (function () {
    function NgCanvasComponent(elementRef) {
        this.elementRef = elementRef;
        this.htmlElement = this.elementRef.nativeElement;
        this.host = d3.select(this.htmlElement);
    }
    NgCanvasComponent.prototype.ngOnInit = function () {
        this.canvas.appendTo(this.host.select('div'));
        if (!this.canvas.hasObservable) {
            this.canvas.buildGroup();
        }
    };
    NgCanvasComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ng-ax-path',
                    encapsulation: core.ViewEncapsulation.None,
                    template: '<div class="ng-ax-path-svg"></div>',
                },] },
    ];
    /** @nocollapse */
    NgCanvasComponent.ctorParameters = function () { return [
        { type: core.ElementRef, },
    ]; };
    NgCanvasComponent.propDecorators = {
        'canvas': [{ type: core.Input },],
    };
    return NgCanvasComponent;
}());

var NgCanvasModule = (function () {
    function NgCanvasModule() {
    }
    NgCanvasModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule],
                    declarations: [
                        NgCanvasComponent
                    ],
                    exports: [
                        NgCanvasComponent
                    ]
                },] },
    ];
    /** @nocollapse */
    NgCanvasModule.ctorParameters = function () { return []; };
    return NgCanvasModule;
}());

exports.Canvas = Canvas;
exports.SvgElement = SvgElement;
exports.Rect = Rect;
exports.Axis = Axis;
exports.Path = Path;
exports.Legend = Legend;
exports.Point = Point;
exports.CanvasStyles = CanvasStyles;
exports.NgCanvasModule = NgCanvasModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
