var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as d3 from 'd3';
import { SvgElement, Rect, SVG_ELEMENT_TYPE } from './svg-element';
import { d3_util } from './d3.util';
var Axis = (function (_super) {
    __extends(Axis, _super);
    function Axis(type, name, unit, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, type, name) || this;
        _this.defaultOptions = {
            tickInterval: 0,
            extraSpace: 0,
            disableAxisTitle: false,
            disableAxisLine: false,
            disableTickLine: false,
            disableTickMark: false,
            disableTickNumber: false
        };
        _this.options = {};
        _this.domain = [0, 0];
        _this.unit = unit;
        Object.assign(_this.options, options);
        return _this;
    }
    Axis.prototype.setRange = function (r) {
        this.range = r;
    };
    Axis.prototype.setDomain = function (domain) {
        this.domain[0] = Math.min(this.domain[0], domain[0]);
        this.domain[1] = Math.max(this.domain[1], domain[1]);
        this.validDomain();
        this.calculateTickValues();
    };
    Axis.prototype.resetDomainWithPoint = function (first, last) {
        var min, max;
        if (this.type === SVG_ELEMENT_TYPE.AXIS_X) {
            min = first.x;
            max = last.x + (this.options.extraSpace || 0);
        }
        else {
            min = first.y;
            max = last.y + (this.options.extraSpace || 0);
        }
        max = Math.floor(max + 0.5);
        this.setDomain([min, max]);
    };
    Object.defineProperty(Axis.prototype, "title", {
        get: function () {
            var title = this.name;
            if (this.unit && this.unit !== '') {
                title = title + " (" + this.unit + ")";
            }
            return title;
        },
        enumerable: true,
        configurable: true
    });
    Axis.prototype.getScale = function () {
        return this.scale;
    };
    Axis.prototype.applyAxisOptions = function (options) {
        this.options = Object.assign({}, options, this.options);
    };
    Axis.prototype.calculateTickValues = function () {
        var domain = this.domain;
        var tickInterval = this.options.tickInterval;
        if (domain) {
            if (tickInterval && tickInterval > 0) {
                this.tickValues = new Array();
                var i = domain[0];
                for (; i <= domain[1]; i = i + tickInterval) {
                    this.tickValues.push(i);
                }
                if (i > domain[1]) {
                    this.tickValues.push(domain[1]);
                }
            }
        }
    };
    Axis.prototype.validDomain = function () {
        var domain = this.domain;
        if (domain && domain[0] > domain[1]) {
            this.domain = (_a = [domain[1], domain[0]], domain[0] = _a[0], domain[1] = _a[1], _a);
        }
        var _a;
    };
    Axis.prototype.createAxisAndScale = function () {
        // scale
        this.scale = d3.scaleLinear()
            .domain(this.domain)
            .range(this.range);
        // axis
        this.axis = this.type === SVG_ELEMENT_TYPE.AXIS_X
            ? d3.axisBottom(this.scale)
            : d3.axisLeft(this.scale);
        // tick
        if (this.tickValues && this.tickValues.length > 0) {
            this.axis.tickValues(this.tickValues);
        }
    };
    Axis.prototype.appendAxisTitle = function () {
        var parentSize = this.parent.size;
        var className = this.type.substr(-1).toLocaleLowerCase() + "-axis-title";
        var text = this.group
            .append('text')
            .attr('class', "axis-title " + className)
            .text(this.title);
        if (this.type === SVG_ELEMENT_TYPE.AXIS_X) {
            text
                .attr('x', Rect.width(parentSize))
                .attr('y', -3)
                .attr('dy', '-.35em')
                .attr('fill', '#000')
                .style('text-anchor', 'end');
        }
        else {
            text
                .attr('dx', 8)
                .attr('dy', '.8em')
                .attr('fill', '#000')
                .style('text-anchor', 'start');
        }
    };
    Axis.prototype.appendTickLine = function (noZeroTick, hasLastTick) {
        if (noZeroTick === void 0) { noZeroTick = true; }
        if (hasLastTick === void 0) { hasLastTick = true; }
        var parentSize = this.parent.size;
        var tickClass = noZeroTick ? '.tick:not(:first-of-type)' : '.tick';
        var lines = this.group.selectAll(tickClass)
            .append('line')
            .attr('stroke', '#777')
            .attr('stroke-dasharray', '2,2');
        if (this.type === SVG_ELEMENT_TYPE.AXIS_X) {
            lines.attr('y2', -Rect.height(parentSize));
        }
        else {
            lines.attr('x2', Rect.width(parentSize));
        }
    };
    Axis.prototype.buildGroup = function () {
        this.createAxisAndScale();
        var parentSize = this.parent.size;
        var className = this.type.substr(-1).toLocaleLowerCase() + "-axis";
        var noZeroTick = true;
        this.group
            .attr('class', "axis " + className)
            .call(this.axis);
        if (this.type === SVG_ELEMENT_TYPE.AXIS_X) {
            this.group.attr('transform', 'translate(0,' + parentSize.height + ')');
        }
        if (!this.options.disableAxisTitle) {
            this.appendAxisTitle();
        }
        if (this.options.disableAxisLine) {
            this.disableAxisLine();
            this.disableTickMark();
            noZeroTick = false;
        }
        if (this.options.disableTickMark) {
            this.disableTickMark();
        }
        if (!!this.options.disableTickNumber) {
            this.disableTickNumber();
        }
        if (!this.options.disableTickLine) {
            this.appendTickLine(noZeroTick);
        }
    };
    Axis.prototype.getSize = function () {
        return d3_util.getRect(this.group);
    };
    Axis.prototype.disableAxisLine = function () {
        this.group.select('path').attr('stroke', 'none');
    };
    Axis.prototype.disableTickMark = function () {
        this.group.selectAll('.tick').select('line').attr('stroke', 'none');
    };
    Axis.prototype.disableTickNumber = function () {
        this.group.selectAll('.tick').select('text').attr('fill', 'none');
    };
    return Axis;
}(SvgElement));
export { Axis };
//# sourceMappingURL=axis.js.map