import { Observable } from 'rxjs/Observable';
import * as d3 from 'd3';
import { Axis, AxisOptions } from './axis';
import { Path } from './path';
import { Point } from './point';
import { SvgElement, RectangleSize } from './svg-element';
import { Legend, LegendShape } from './legend';
import { CanvasStyle } from './canvas-style';
export interface CanvasOptions {
    hasTitle?: boolean;
    hasLegend?: boolean;
}
export declare class Canvas extends SvgElement {
    /**
   * Canvas margin, can be changed
   */
    margin: RectangleSize;
    defaultMargin: RectangleSize;
    fullSize: RectangleSize;
    private canvasStyle;
    private styleOptions;
    canvasOptions: CanvasOptions;
    legend: Legend;
    private canvasDomain;
    private titleGroup;
    private paths;
    private xyAxis;
    private host;
    private _hasObservable;
    private runningObservable;
    constructor(name: string, fullSize: RectangleSize, canvasStyle?: CanvasStyle, domain?: [number, number], legend?: Legend);
    readonly hasObservable: boolean;
    xAxis: Axis;
    yAxis: Axis;
    /**
     * set legend shape
     * @memberof Canvas
     */
    legendShape: LegendShape;
    getX(): Axis;
    getY(): Axis;
    createX(name: string, unit: string, options: AxisOptions): Axis;
    createY(name: string, unit: string, options: AxisOptions): Axis;
    private setXyAxis(axis);
    /**
     * @public
     * @param {(Path | Point[] | [number, number][] | Observable<[number, number][] | {x: number, y: number}[])} data
     * @returns {Path | null}
     * @memberof Canvas
     * @see Path, Point
     */
    addPath(data: Path | Point[] | [number, number][] | Observable<[number, number][] | {
        x: number;
        y: number;
    }[]>, name?: string): Path | null;
    addPathFromArray(data: [number, number][], name?: string): Path;
    addPathFromPoints(points: Point[], name?: string): Path;
    addPathFromPath(path: Path): Path;
    private _addPathFromObservable(data, name?);
    xScale(value: number): number;
    yScale(value: number): number;
    appendTo(host: d3.Selection<d3.BaseType, {}, null, undefined>): void;
    private setHostSize();
    private applyCanvasStyle();
    private appendGroup();
    buildGroup(): void;
    private buildTitle();
}
