var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Observable } from 'rxjs/Observable';
import { Axis } from './axis';
import { Path } from './path';
import { Point } from './point';
import { SvgElement, Rect, SVG_ELEMENT_TYPE } from './svg-element';
import { LegendStyle } from './legend';
import { d3_util } from './d3.util';
import { CanvasStyle, CanvasStyles } from './canvas-style';
var DEFAULT_MARGIN = { top: 5, left: 30, right: 15, bottom: 20 };
var styles = [
    LegendStyle.UP_LEFT_IN_BLOCK,
    LegendStyle.UP_LEFT_IN_LINE,
    LegendStyle.UP_RIGHT_IN_BLOCK,
    LegendStyle.UP_RIGHT_IN_LINE,
    LegendStyle.BOTTOM_LEFT_IN_BLOCK,
    LegendStyle.BOTTOM_LEFT_IN_LINE,
    LegendStyle.BOTTOM_RIGHT_IN_BLOCK,
    LegendStyle.BOTTOM_RIGHT_IN_LINE,
    LegendStyle.UP_CENTER_IN_LINE,
    LegendStyle.BOTTOM_CENTER_IN_LINE,
    LegendStyle.UP_CENTER_OUT_LINE,
    LegendStyle.UP_LEFT_OUT_LINE,
    LegendStyle.UP_RIGHT_OUT_LINE,
    LegendStyle.BOTTOM_CENTER_OUT_LINE,
    LegendStyle.UP_LEFT_SIDE_BLOCK,
    LegendStyle.UP_RIGHT_SIDE_BLOCK,
    LegendStyle.BOTTOM_LEFT_SIDE_BLOCK,
    LegendStyle.BOTTOM_RIGHT_SIDE_BLOCK
];
var Canvas = (function (_super) {
    __extends(Canvas, _super);
    function Canvas(name, fullSize, canvasStyle, domain, legend) {
        if (canvasStyle === void 0) { canvasStyle = CanvasStyle.Coordinate; }
        if (domain === void 0) { domain = [0, 0]; }
        if (legend === void 0) { legend = LegendStyle.UP_LEFT_IN_BLOCK; }
        var _this = _super.call(this, SVG_ELEMENT_TYPE.CANVAS, name) || this;
        /**
       * Canvas margin, can be changed
       */
        _this.margin = { top: 0, left: 0, right: 0, bottom: 0 };
        _this.defaultMargin = DEFAULT_MARGIN;
        _this.fullSize = { width: 0, height: 0 };
        _this.canvasOptions = {};
        _this.paths = new Array();
        _this.xyAxis = {};
        _this._hasObservable = false;
        // TODO
        _this.runningObservable = 0;
        // this.legend = styles[10];
        _this.size = { width: 0, height: 0 };
        _this.fullSize = fullSize;
        _this.canvasStyle = canvasStyle || CanvasStyle.Coordinate;
        _this.styleOptions = CanvasStyles[_this.canvasStyle];
        _this.legend = legend;
        _this.canvasDomain = domain;
        _this.titleGroup = d3_util.createGroup();
        return _this;
    }
    Object.defineProperty(Canvas.prototype, "hasObservable", {
        get: function () {
            return this._hasObservable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "xAxis", {
        get: function () {
            if (!this.xyAxis[SVG_ELEMENT_TYPE.AXIS_X]) {
                this.xAxis = this.createX('', '', {});
            }
            return this.xyAxis[SVG_ELEMENT_TYPE.AXIS_X];
        },
        set: function (xAxis) {
            if (xAxis) {
                xAxis.type = SVG_ELEMENT_TYPE.AXIS_X;
                this.setXyAxis(xAxis);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "yAxis", {
        get: function () {
            if (!this.xyAxis[SVG_ELEMENT_TYPE.AXIS_Y]) {
                this.yAxis = this.createY('', '', {});
            }
            return this.xyAxis[SVG_ELEMENT_TYPE.AXIS_Y];
        },
        set: function (yAxis) {
            if (yAxis) {
                yAxis.type = SVG_ELEMENT_TYPE.AXIS_Y;
                this.setXyAxis(yAxis);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Canvas.prototype, "legendShape", {
        /**
         * set legend shape
         * @memberof Canvas
         */
        set: function (legendShape) {
            this.legend.legendShape = legendShape;
        },
        enumerable: true,
        configurable: true
    });
    Canvas.prototype.getX = function () {
        return this.xAxis;
    };
    Canvas.prototype.getY = function () {
        return this.yAxis;
    };
    Canvas.prototype.createX = function (name, unit, options) {
        return this.xAxis = new Axis(SVG_ELEMENT_TYPE.AXIS_X, name, unit, options);
    };
    Canvas.prototype.createY = function (name, unit, options) {
        return this.yAxis = new Axis(SVG_ELEMENT_TYPE.AXIS_Y, name, unit, options);
    };
    Canvas.prototype.setXyAxis = function (axis) {
        this.xyAxis[axis.type] = axis;
        this.addChild(axis);
    };
    /**
     * @public
     * @param {(Path | Point[] | [number, number][] | Observable<[number, number][] | {x: number, y: number}[])} data
     * @returns {Path | null}
     * @memberof Canvas
     * @see Path, Point
     */
    Canvas.prototype.addPath = function (data, name) {
        if (data instanceof Path) {
            return this.addPathFromPath(data);
        }
        else if (data instanceof Array && data.length > 0) {
            if (data[0] instanceof Point) {
                return this.addPathFromPoints(data, name);
            }
            else if (data[0] instanceof Array) {
                return this.addPathFromArray(data, name);
            }
        }
        else if (data instanceof Observable) {
            return this._addPathFromObservable(data, name);
        }
        return null;
    };
    Canvas.prototype.addPathFromArray = function (data, name) {
        return this.addPathFromPoints(data.map(function (d) { return new Point(d[0], d[1]); }), name);
    };
    Canvas.prototype.addPathFromPoints = function (points, name) {
        return this.addPathFromPath(new Path(points, name));
    };
    Canvas.prototype.addPathFromPath = function (path) {
        this.paths.push(path);
        this.addChild(path);
        return path;
    };
    Canvas.prototype._addPathFromObservable = function (data, name) {
        var _this = this;
        this._hasObservable = true;
        // TODO
        this.runningObservable++;
        var path = new Path([], name);
        data.subscribe(function (d) {
            var points;
            if (d[0] instanceof Array) {
                points = d.map(function (p) { return new Point(p[0], p[1]); });
            }
            else {
                points = d.map(function (p) { return new Point(p.x, p.y); });
            }
            path.setPoints(points);
            _this.addPathFromPath(path);
            // TODO
            _this.runningObservable--;
            if (_this.runningObservable === 0) {
                _this.buildGroup();
            }
        });
        return path;
    };
    Canvas.prototype.xScale = function (value) {
        return this.xAxis.getScale()(value);
    };
    Canvas.prototype.yScale = function (value) {
        return this.yAxis.getScale()(value);
    };
    Canvas.prototype.appendTo = function (host) {
        this.host = host.insert('svg');
        this.setHostSize();
        this.appendGroup();
    };
    Canvas.prototype.setHostSize = function () {
        this.host.attr('width', Rect.width(this.fullSize))
            .attr('height', Rect.height(this.fullSize));
    };
    Canvas.prototype.applyCanvasStyle = function () {
        this.canvasOptions = Object.assign({}, this.styleOptions.CanvasOptions, this.canvasOptions);
    };
    Canvas.prototype.appendGroup = function () {
        var _this = this;
        this.applyCanvasStyle();
        this.host.append(function () { return _this.group.node(); });
        if (this.canvasOptions.hasLegend) {
            this.host.append(function () { return _this.legend.group.node(); });
        }
        if (this.canvasOptions.hasTitle) {
            this.host.append(function () { return _this.titleGroup.node(); });
        }
    };
    Canvas.prototype.buildGroup = function () {
        var _this = this;
        var xAxis = this.xAxis;
        var yAxis = this.yAxis;
        this.applyCanvasStyle();
        // draw title
        if (this.canvasOptions.hasTitle) {
            this.buildTitle();
        }
        // draw legend items
        if (this.canvasOptions.hasLegend) {
            var legendItem_1 = this.legend.createLegendItem();
            this.paths.forEach(function (p, i) { return legendItem_1.loop(p.name, p.color, i); });
            legendItem_1.end();
        }
        // calculate margin
        var margin = this.canvasOptions.hasLegend ? this.legend.getCoordinateMargin(this.margin) : this.margin;
        var defMargin = this.defaultMargin;
        var titleRect = d3_util.getRect(this.titleGroup);
        var chartMargin = {
            left: Rect.left(defMargin) + Rect.left(margin),
            top: Rect.top(defMargin) + Rect.top(margin) + titleRect.height,
            right: Rect.right(defMargin) + Rect.right(margin),
            bottom: Rect.bottom(defMargin) + Rect.bottom(margin)
        };
        // move canvas
        this.group.attr('transform', "translate(" + chartMargin.left + "," + chartMargin.top + ")");
        // calculate canvas size
        this.size.width = Rect.width(this.fullSize) - chartMargin.left - chartMargin.right;
        this.size.height = Rect.height(this.fullSize) - chartMargin.top - chartMargin.bottom;
        xAxis.applyAxisOptions(this.styleOptions.AxisOptions);
        yAxis.applyAxisOptions(this.styleOptions.AxisOptions);
        // set x and y axis
        xAxis.setRange([0, this.size.width]);
        xAxis.setDomain([0, this.canvasDomain[0]]);
        yAxis.setRange([this.size.height, 0]);
        yAxis.setDomain([0, this.canvasDomain[1]]);
        this.paths.forEach(function (p) {
            xAxis.resetDomainWithPoint(p.xFirst, p.xLast);
            yAxis.resetDomainWithPoint(p.yFirst, p.yLast);
        });
        // draw x-axis, y-axis
        [xAxis, yAxis].forEach(function (axis) { return _this.rebuild(axis); });
        // draw paths
        this.paths.forEach(function (path) { return _this.rebuild(path); });
        // Title position
        if (this.canvasOptions.hasTitle) {
            var xTitle = chartMargin.left + (Rect.width(this.size) - titleRect.width) / 2;
            this.titleGroup.attr('transform', "translate(" + xTitle + "," + titleRect.height + ")");
        }
        // Legend position
        if (this.canvasOptions.hasLegend) {
            this.legend.transform(this.size, this.fullSize, titleRect);
        }
    };
    Canvas.prototype.buildTitle = function () {
        // build title
        this.titleGroup.selectAll('*').remove();
        this.titleGroup.append('text')
            .attr('fill', '#000')
            .style('font', '18px sans-serif')
            .text(this.name);
    };
    return Canvas;
}(SvgElement));
export { Canvas };
//# sourceMappingURL=canvas.js.map