import { SvgElement, RectangleSize } from './svg-element';
export declare enum LegendLayout {
    BLOCK = "Block",
    LINE = "Line",
}
export declare enum LegendPosition {
    UP_LEFT = "UpLeft",
    UP_RIGHT = "UpRight",
    BOTTOM_LEFT = "BottomLeft",
    BOTTOM_RIGHT = "BottomRight",
    UP_CENTER = "UpCenter",
    BOTTOM_CENTER = "BottomCenter",
}
export declare enum LegendRange {
    IN = "In",
    OUT = "Out",
    SIDE = "Side",
}
export declare enum LegendShape {
    LINE = "Line",
    CIRCLE = "Circle",
    RECTANGLE = "Rectangle",
    ELLIPSE = "Ellipse",
}
export declare class Legend extends SvgElement {
    layout: LegendLayout;
    position: LegendPosition;
    range: LegendRange;
    private positionH;
    private positionV;
    private legendSize;
    legendShape: LegendShape;
    /**
     * Legend margin, can be changed
     */
    private insideRect;
    constructor(layout: LegendLayout, position: LegendPosition, range: LegendRange);
    readonly $legendSize: ClientRect;
    buildGroup(): void;
    private itemX(i, offset);
    private itemY(i);
    private x(size, fullSize, titleSize);
    private y(size, fullSize, titleSize);
    getCoordinateMargin(coordinateMargin: RectangleSize): RectangleSize;
    createLegendItem(): {
        loop: Function;
        end: Function;
    };
    transform(size: RectangleSize, fullSize: RectangleSize, titleSize: RectangleSize): void;
    private createItemFigure(color);
    private getItemX(figureRect);
}
export declare namespace LegendStyle {
    const UP_LEFT_IN_BLOCK: Legend;
    const UP_LEFT_IN_LINE: Legend;
    const UP_RIGHT_IN_BLOCK: Legend;
    const UP_RIGHT_IN_LINE: Legend;
    const BOTTOM_LEFT_IN_BLOCK: Legend;
    const BOTTOM_LEFT_IN_LINE: Legend;
    const BOTTOM_RIGHT_IN_BLOCK: Legend;
    const BOTTOM_RIGHT_IN_LINE: Legend;
    const UP_CENTER_IN_LINE: Legend;
    const BOTTOM_CENTER_IN_LINE: Legend;
    const UP_CENTER_OUT_LINE: Legend;
    const UP_LEFT_OUT_LINE: Legend;
    const UP_RIGHT_OUT_LINE: Legend;
    const BOTTOM_CENTER_OUT_LINE: Legend;
    const UP_LEFT_SIDE_BLOCK: Legend;
    const BOTTOM_LEFT_SIDE_BLOCK: Legend;
    const UP_RIGHT_SIDE_BLOCK: Legend;
    const BOTTOM_RIGHT_SIDE_BLOCK: Legend;
}
