var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { SvgElement, Rect, SVG_ELEMENT_TYPE } from './svg-element';
import { d3_util } from './d3.util';
var PositionH;
(function (PositionH) {
    PositionH["LEFT"] = "Left";
    PositionH["RIGHT"] = "Right";
    PositionH["CENTER"] = "Center";
})(PositionH || (PositionH = {}));
var PositionV;
(function (PositionV) {
    PositionV["UP"] = "Up";
    PositionV["BOTTOM"] = "Bottom";
    PositionV["MIDDLE"] = "Middle";
})(PositionV || (PositionV = {}));
export var LegendLayout;
(function (LegendLayout) {
    LegendLayout["BLOCK"] = "Block";
    LegendLayout["LINE"] = "Line";
})(LegendLayout || (LegendLayout = {}));
export var LegendPosition;
(function (LegendPosition) {
    LegendPosition["UP_LEFT"] = "UpLeft";
    LegendPosition["UP_RIGHT"] = "UpRight";
    LegendPosition["BOTTOM_LEFT"] = "BottomLeft";
    LegendPosition["BOTTOM_RIGHT"] = "BottomRight";
    LegendPosition["UP_CENTER"] = "UpCenter";
    LegendPosition["BOTTOM_CENTER"] = "BottomCenter";
})(LegendPosition || (LegendPosition = {}));
export var LegendRange;
(function (LegendRange) {
    LegendRange["IN"] = "In";
    LegendRange["OUT"] = "Out";
    LegendRange["SIDE"] = "Side";
})(LegendRange || (LegendRange = {}));
export var LegendShape;
(function (LegendShape) {
    LegendShape["LINE"] = "Line";
    LegendShape["CIRCLE"] = "Circle";
    LegendShape["RECTANGLE"] = "Rectangle";
    LegendShape["ELLIPSE"] = "Ellipse";
})(LegendShape || (LegendShape = {}));
var yAxisWidth = 20;
var Legend = (function (_super) {
    __extends(Legend, _super);
    function Legend(layout, position, range) {
        var _this = _super.call(this, SVG_ELEMENT_TYPE.LEGEND) || this;
        _this.layout = layout;
        _this.position = position;
        _this.range = range;
        _this.legendShape = LegendShape.LINE;
        /**
         * Legend margin, can be changed
         */
        _this.insideRect = { top: 30, right: 25, bottom: 20, left: 30 };
        var ps = position.split(/(?=[A-Z])/);
        _this.positionV = PositionV[ps[0].toUpperCase()];
        _this.positionH = PositionH[ps[1].toUpperCase()];
        return _this;
    }
    Object.defineProperty(Legend.prototype, "$legendSize", {
        get: function () {
            return this.legendSize;
        },
        enumerable: true,
        configurable: true
    });
    Legend.prototype.buildGroup = function () {
        this.group
            .attr('class', 'legend')
            .style('fill', 'transparent')
            .style('stroke-width', '1px');
    };
    Legend.prototype.itemX = function (i, offset) {
        switch (this.layout) {
            case LegendLayout.LINE:
                return 15 * i + offset;
            case LegendLayout.BLOCK:
            default:
                return 0;
        }
    };
    Legend.prototype.itemY = function (i) {
        switch (this.layout) {
            case LegendLayout.LINE:
                return 0;
            case LegendLayout.BLOCK:
            default:
                return 15 * i;
        }
    };
    Legend.prototype.x = function (size, fullSize, titleSize) {
        if (this.range === LegendRange.IN || this.range === LegendRange.OUT) {
            switch (this.positionH) {
                case PositionH.RIGHT:
                    return Rect.width(fullSize) - this.legendSize.width - Rect.right(this.insideRect);
                case PositionH.CENTER:
                    return (Rect.width(size) - this.legendSize.width) / 2 + Rect.left(this.insideRect) + yAxisWidth;
                case PositionH.LEFT:
                default:
                    return Rect.left(this.insideRect) + yAxisWidth;
            }
        }
        else {
            // LegendRange.SIDE
            switch (this.positionH) {
                case PositionH.RIGHT:
                    return Rect.width(size) + this.legendSize.width / 2;
                case PositionH.LEFT:
                default:
                    return yAxisWidth;
            }
        }
    };
    Legend.prototype.y = function (size, fullSize, titleSize) {
        if (this.range === LegendRange.IN || this.range === LegendRange.SIDE) {
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    return Rect.height(size) - this.legendSize.height;
                case PositionV.UP:
                default:
                    return this.insideRect.top + Rect.height(titleSize);
            }
        }
        else {
            // LegendRange.OUT
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    return Rect.height(fullSize) - this.legendSize.height - 10;
                case PositionV.UP:
                default:
                    return Rect.height(titleSize) + Rect.top(this.insideRect) / 2;
            }
        }
    };
    Legend.prototype.getCoordinateMargin = function (coordinateMargin) {
        var margin = Object.assign({}, coordinateMargin);
        if (this.range === LegendRange.OUT) {
            switch (this.positionV) {
                case PositionV.BOTTOM:
                    margin.bottom = margin.bottom || 0 + 30;
                    break;
                case PositionV.UP:
                default:
                    margin.top = margin.top || 0 + 30;
            }
        }
        else if (this.range === LegendRange.SIDE) {
            switch (this.positionH) {
                case PositionH.RIGHT:
                    margin.right += this.legendSize.width + Rect.right(this.insideRect);
                    break;
                case PositionH.LEFT:
                default:
                    margin.left += this.legendSize.width + Rect.left(this.insideRect);
            }
        }
        return margin;
    };
    Legend.prototype.createLegendItem = function () {
        var _this = this;
        this.clear();
        var offset = 0;
        return {
            loop: function (name, color, index) {
                var legendItem = d3_util.createGroup().attr('class', 'legend-item');
                var figure = _this.createItemFigure(color);
                _this.group.append(function () { return legendItem.node(); });
                legendItem.append(function () { return figure.node(); });
                var figureRect = d3_util.getRect(figure);
                legendItem.append('text')
                    .attr('x', _this.getItemX(figureRect))
                    .attr('y', figureRect.height === 0 ? 3 : figureRect.height / 2)
                    .attr('fill', '#000')
                    .style('font', '10px sans-serif')
                    .text(name);
                var xItem = _this.itemX(index, offset);
                var yItem = _this.itemY(index);
                legendItem.attr('transform', "translate(" + xItem + "," + yItem + ")");
                offset += d3_util.getRect(legendItem).width;
                return legendItem;
            },
            end: function () {
                _this.legendSize = d3_util.getRect(_this.group);
            }
        };
    };
    Legend.prototype.transform = function (size, fullSize, titleSize) {
        var xLegend = this.x(size, fullSize, titleSize);
        var yLegend = this.y(size, fullSize, titleSize);
        this.group.attr('transform', "translate(" + xLegend + "," + yLegend + ")");
        this.legendSize = d3_util.getRect(this.group);
    };
    Legend.prototype.createItemFigure = function (color) {
        switch (this.legendShape) {
            case LegendShape.CIRCLE:
                return d3_util.createGroup('circle')
                    .attr('cy', 1)
                    .attr('r', 4)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case LegendShape.RECTANGLE:
                return d3_util.createGroup('rect')
                    .attr('y', -3)
                    .attr('width', 10)
                    .attr('height', 5)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case LegendShape.ELLIPSE:
                return d3_util.createGroup('ellipse')
                    .attr('y', -2)
                    .attr('rx', 8)
                    .attr('ry', 4)
                    .style('stroke', 'transparent')
                    .style('fill', color);
            case LegendShape.LINE:
            default:
                return d3_util.createGroup('line')
                    .attr('x2', 10)
                    .attr('y2', 0)
                    .style('stroke', color)
                    .style('fill', 'transparent')
                    .style('stroke-width', '1px');
        }
    };
    Legend.prototype.getItemX = function (figureRect) {
        switch (this.legendShape) {
            case LegendShape.CIRCLE:
            case LegendShape.ELLIPSE:
                return figureRect.width / 2 + 3;
            case LegendShape.RECTANGLE:
            case LegendShape.LINE:
            default:
                return figureRect.width + 3;
        }
    };
    return Legend;
}(SvgElement));
export { Legend };
export var LegendStyle;
(function (LegendStyle) {
    // in & up
    LegendStyle.UP_LEFT_IN_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.UP_LEFT, LegendRange.IN);
    LegendStyle.UP_LEFT_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_LEFT, LegendRange.IN);
    LegendStyle.UP_RIGHT_IN_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.UP_RIGHT, LegendRange.IN);
    LegendStyle.UP_RIGHT_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_RIGHT, LegendRange.IN);
    // in & bottom
    LegendStyle.BOTTOM_LEFT_IN_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.BOTTOM_LEFT, LegendRange.IN);
    LegendStyle.BOTTOM_LEFT_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.BOTTOM_LEFT, LegendRange.IN);
    LegendStyle.BOTTOM_RIGHT_IN_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.BOTTOM_RIGHT, LegendRange.IN);
    LegendStyle.BOTTOM_RIGHT_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.BOTTOM_RIGHT, LegendRange.IN);
    LegendStyle.UP_CENTER_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_CENTER, LegendRange.IN);
    LegendStyle.BOTTOM_CENTER_IN_LINE = new Legend(LegendLayout.LINE, LegendPosition.BOTTOM_CENTER, LegendRange.IN);
    // out & line
    LegendStyle.UP_CENTER_OUT_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_CENTER, LegendRange.OUT);
    LegendStyle.UP_LEFT_OUT_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_LEFT, LegendRange.OUT);
    LegendStyle.UP_RIGHT_OUT_LINE = new Legend(LegendLayout.LINE, LegendPosition.UP_RIGHT, LegendRange.OUT);
    LegendStyle.BOTTOM_CENTER_OUT_LINE = new Legend(LegendLayout.LINE, LegendPosition.BOTTOM_CENTER, LegendRange.OUT);
    // side & block
    LegendStyle.UP_LEFT_SIDE_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.UP_LEFT, LegendRange.SIDE);
    LegendStyle.BOTTOM_LEFT_SIDE_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.BOTTOM_LEFT, LegendRange.SIDE);
    LegendStyle.UP_RIGHT_SIDE_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.UP_RIGHT, LegendRange.SIDE);
    LegendStyle.BOTTOM_RIGHT_SIDE_BLOCK = new Legend(LegendLayout.BLOCK, LegendPosition.BOTTOM_RIGHT, LegendRange.SIDE);
})(LegendStyle || (LegendStyle = {}));
//# sourceMappingURL=legend.js.map