var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { CSPL } from './cspl';
import { SvgElement, SVG_ELEMENT_TYPE } from './svg-element';
import { PointShape } from './point';
export var PathType;
(function (PathType) {
    PathType["CURVED_LINE"] = "CurvedLine";
    PathType["STRAIGHT_LINE"] = "StraightLine";
})(PathType || (PathType = {}));
var Path = (function (_super) {
    __extends(Path, _super);
    function Path(points, name) {
        if (name === void 0) { name = ''; }
        var _this = _super.call(this, SVG_ELEMENT_TYPE.PATH, name) || this;
        _this.axisStep = 1;
        _this.hasDot = true;
        _this.pathType = PathType.CURVED_LINE;
        _this.pointShape = PointShape.CIRCLE;
        _this.xRef = {};
        _this.yRef = {};
        _this.points = points;
        _this.setDefaultStyle();
        _this.sort();
        return _this;
    }
    Path.prototype.sort = function () {
        var points = this.points.slice();
        this.points.sort(function (a, b) { return a.x - b.x; });
        points.sort(function (a, b) { return a.y - b.y; });
        this.xRef.first = this.points[0];
        this.xRef.last = this.points[this.points.length - 1];
        this.yRef.first = points[0];
        this.yRef.last = points[points.length - 1];
    };
    Object.defineProperty(Path.prototype, "color", {
        get: function () {
            return this.getStyle('stroke');
        },
        set: function (color) {
            this.setStyle('stroke', color);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "xFirst", {
        get: function () {
            return this.xRef.first;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "xLast", {
        get: function () {
            return this.xRef.last;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "yFirst", {
        get: function () {
            return this.yRef.first;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Path.prototype, "yLast", {
        get: function () {
            return this.yRef.last;
        },
        enumerable: true,
        configurable: true
    });
    Path.prototype.setPoints = function (points) {
        this.points = points;
        this.sort();
    };
    Path.prototype.setDefaultStyle = function () {
        this.setAttr('class', 'line')
            .setStyle('stroke', '#000')
            .setAttr('fill', 'transparent')
            .setAttr('stroke-width', '1.5px');
    };
    Path.prototype.buildGroup = function () {
        var _this = this;
        this.applyStyle();
        var path;
        if (this.pathType === PathType.CURVED_LINE) {
            path = this._buildCurvedPath();
        }
        else {
            path = this.points.map(function (p) { return _this._toXY(p.x, p.y); });
        }
        this.group.append('path').attr('d', path.reduce(function (s, p) { return s += p.join(',') + ' '; }, 'M').trim());
        if (this.hasDot) {
            this.points.forEach(function (p) { return _this.group.append(function () { return p.createElement(_this.parent, _this.color, _this.pointShape).node(); }); });
        }
    };
    Path.prototype._buildCurvedPath = function () {
        var path = [];
        var step = 2 / this.parent.xScale(this.axisStep);
        var points = this.points;
        var xs = [], ys = [], ks = [];
        points.forEach(function (p, i) {
            xs[i] = p.x;
            ys[i] = p.y;
            ks[i] = 1;
        });
        CSPL.getNaturalKs(xs, ys, ks);
        var minx = points[0].x;
        var maxx = points[points.length - 1].x;
        path.push(this._toXY(minx, CSPL.evalSpline(minx, xs, ys, ks)));
        for (var i = minx + step; i <= maxx; i += step) {
            path.push(this._toXY(i, CSPL.evalSpline(i, xs, ys, ks)));
        }
        return path;
    };
    Path.prototype._toXY = function (x, y) {
        return [this.parent.xScale(x), this.parent.yScale(y)];
    };
    return Path;
}(SvgElement));
export { Path };
//# sourceMappingURL=path.js.map