import { d3_util } from './d3.util';
var Point = (function () {
    function Point(x, y, color) {
        if (color === void 0) { color = ''; }
        this.x = x;
        this.y = y;
        this.pointShape = null;
        this.color = color;
    }
    Point.prototype.createElement = function (canvas, color, pointShape) {
        var element;
        var x = canvas.xScale(this.x);
        var y = canvas.yScale(this.y);
        pointShape = this.pointShape || pointShape;
        color = this.color || color;
        if (pointShape === PointShape.SQUARE) {
            var squareLength = 5;
            element = d3_util.createGroup('rect')
                .attr('x', x - squareLength / 2).attr('y', y - squareLength / 2)
                .attr('width', squareLength).attr('height', squareLength);
        }
        else if (pointShape === PointShape.TRIANGLE) {
            var sideLength = 8;
            var pts = d3_util.createEqTriangle(sideLength, [x, y]);
            element = d3_util.createGroup('polygon')
                .attr('points', pts.map(function (p) { return p.join(','); }).join(' '));
        }
        else {
            var radius = 3;
            element = d3_util.createGroup('circle')
                .attr('cx', x)
                .attr('cy', y)
                .attr('r', radius);
        }
        element.attr('fill', color).attr('stroke', 'transparent');
        return element;
    };
    return Point;
}());
export { Point };
export var PointShape;
(function (PointShape) {
    PointShape["CIRCLE"] = "Circle";
    PointShape["TRIANGLE"] = "Triangle";
    PointShape["SQUARE"] = "Square";
})(PointShape || (PointShape = {}));
//# sourceMappingURL=point.js.map