import { d3_util } from './d3.util';
export var SVG_ELEMENT_TYPE;
(function (SVG_ELEMENT_TYPE) {
    SVG_ELEMENT_TYPE["CANVAS"] = "Canvas";
    SVG_ELEMENT_TYPE["AXIS_X"] = "AxisX";
    SVG_ELEMENT_TYPE["AXIS_Y"] = "AxisY";
    SVG_ELEMENT_TYPE["LEGEND"] = "Legend";
    SVG_ELEMENT_TYPE["PATH"] = "Path";
})(SVG_ELEMENT_TYPE || (SVG_ELEMENT_TYPE = {}));
var SvgElement = (function () {
    function SvgElement(type, name) {
        if (name === void 0) { name = ''; }
        this.children = new Array();
        this.attrs = {};
        this.styles = {};
        this.type = type;
        this.name = name;
        this.group = d3_util.createGroup();
    }
    SvgElement.prototype.setAttr = function (name, value) {
        this.attrs[name] = value;
        return this;
    };
    SvgElement.prototype.delAttr = function (name) {
        delete this.attrs[name];
    };
    SvgElement.prototype.setStyle = function (name, value) {
        this.styles[name] = value;
        return this;
    };
    SvgElement.prototype.getStyle = function (name) {
        return this.styles[name];
    };
    SvgElement.prototype.delStyle = function (name) {
        delete this.styles[name];
    };
    SvgElement.prototype.addChild = function (child) {
        if (child) {
            child.parent = this;
            this.children.push(child);
        }
        return this;
    };
    SvgElement.prototype.applyStyle = function () {
        var _this = this;
        Object.keys(this.styles).forEach(function (k) { return _this.group.style(k, _this.styles[k]); });
        Object.keys(this.attrs).forEach(function (k) { return _this.group.attr(k, _this.attrs[k]); });
    };
    SvgElement.prototype.clearAll = function () {
        this.group.selectAll('*').remove();
        this.group.remove();
    };
    SvgElement.prototype.clear = function () {
        this.group.selectAll('*').remove();
    };
    SvgElement.prototype.rebuild = function (child) {
        child.clearAll();
        this.group.append(function () { return child.group.node(); });
        child.buildGroup();
    };
    return SvgElement;
}());
export { SvgElement };
export var Rect = {
    left: function (o) { return o ? o.left || 0 : 0; },
    top: function (o) { return o ? o.top || 0 : 0; },
    right: function (o) { return o ? o.right || 0 : 0; },
    bottom: function (o) { return o ? o.bottom || 0 : 0; },
    width: function (o) { return o ? o.width || 0 : 0; },
    height: function (o) { return o ? o.height || 0 : 0; }
};
//# sourceMappingURL=svg-element.js.map